// -*- C++ -*-
/*!
 * @file  RTC_RobotMotion_Manual.h
 * @brief ゲームコントローラのデータよりRobotMotion生成　
 * @date  $Date$
 *
 * @author AIZUK
 * kohno@aizuk.jp
 *
 * $Id$
 */

#ifndef RTC_ROBOTMOTION_MANUAL_H
#define RTC_ROBOTMOTION_MANUAL_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <math.h>

/*!
 * @class RTC_RobotMotion_Manual
 * @brief ゲームコントローラのデータよりRobotMotion生成　
 *
 * GamePADのデータをクローラロボットの指令値へ
 * Velocity（ロボット目標速度 vx,va)
 * Sub_CrawlerAngle(各サブクローラ目標角度)
 * MODE(機体動作モード)
 * MOT_Main(左右輪PWM値を直接入力)
 * MOT_Sub(各サブクローラPWM値を直接入力)
 *
 * 入力
 * 　Controller_Type : TimedString
 * 　Button：TimedUlong
 * 　Analog：TimedDoubleSeq
 * 出力
 * 　Velocity　: TimedVelocity2D
 * 　　Sub_CrawlerAngle :  TimedDoubleSeq
 * MODE :  TimedLongSeq
 * MOT_Main :  TimedLongSeq
 * MOT_Sub :  TimedLongSeq
 *
 */
class RTC_RobotMotion_Manual
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  RTC_RobotMotion_Manual(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~RTC_RobotMotion_Manual();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * 
   * - Name:  Velocity_Gain
   * - DefaultValue: 0.36
   */
  double m_Velocity_Gain;
  /*!
   * 
   * - Name:  Omega_Gain
   * - DefaultValue: 2.2
   */
  double m_Omega_Gain;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::TimedString m_Controller_Type;
  /*!
   */
  InPort<RTC::TimedString> m_Controller_TypeIn;
  RTC::TimedULong m_Button;
  /*!
   */
  InPort<RTC::TimedULong> m_ButtonIn;
  RTC::TimedDoubleSeq m_Analog;
  /*!
   */
  InPort<RTC::TimedDoubleSeq> m_AnalogIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  RTC::TimedVelocity2D m_Velocity;
  /*!
   */
  OutPort<RTC::TimedVelocity2D> m_VelocityOut;
  RTC::TimedDoubleSeq m_SubCrawlerAngle;
  /*!
   */
  OutPort<RTC::TimedDoubleSeq> m_SubCrawlerAngleOut;
  RTC::TimedLongSeq m_Mode;
  /*!
   */
  OutPort<RTC::TimedLongSeq> m_ModeOut;
  RTC::TimedLongSeq m_MOT_Main;
  /*!
   */
  OutPort<RTC::TimedLongSeq> m_MOT_MainOut;
  RTC::TimedLongSeq m_MOT_Sub;
  /*!
   */
  OutPort<RTC::TimedLongSeq> m_MOT_SubOut;
  
  RTC::TimedLongSeq m_Scan_Mode;
  /*!
   * LRFスキャン用
   * スキャンフラグ,2D/3D,サーボ動作モード
   */
  OutPort<RTC::TimedLongSeq> m_Scan_ModeOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>
  
///////////////////////////////////
//PS4
//////////////////////////////////
int SQARE;
int OLD_SQARE;
int CROSS;
int CIRCLE;
int OLD_CIRCLE;
int TRIANGLE;
int OLD_TRIANGLE;
int L1;
int R1;
int L2;
int R2;
int SHARE;
int OLD_SHARE;
int OPTION;
int OLD_OPTION;
int L3;
int OLD_L3;
int R3;
int OLD_R3;
int PS;
int PS_CNT=0;
int PS_LP_Lock=0;
int OLD_PS;
int Touch;
int OLD_Touch;
float L2_A;
float R2_A;

//COMMON
float HAT_UP;
float HAT_DOWN;
float HAT_LEFT;
float HAT_RIGHT;
float LEFT_X;
float LEFT_Y;
float LEFT_X_;
float LEFT_Y_;
float RIGHT_X;
float RIGHT_Y;
//--------------------------------//
int p_button[20];//hex->2
int button[20];//hex->2
int A_FLG=0;//
int B_FLG=0;//

 virtual RTC::ReturnCode_t  Button_Rename();
 virtual RTC::ReturnCode_t  Analog_Rename();
//
 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

};


extern "C"
{
  DLL_EXPORT void RTC_RobotMotion_ManualInit(RTC::Manager* manager);
};

#endif // RTC_ROBOTMOTION_MANUAL_H
