// -*- C++ -*-
/*!
 * @file  RTC_RobotMotion_Manual.cpp
 * @brief ゲームコントローラのデータよりRobotMotion生成　
 * @date $Date$
 *
 * @author AIZUK
 * kohno@aizuk.jp
 *
 * $Id$
 */

#include "RTC_RobotMotion_Manual.h"

#define degree		(180 / M_PI)
// Module specification
// <rtc-template block="module_spec">
static const char* rtc_robotmotion_manual_spec[] =
  {
    "implementation_id", "RTC_RobotMotion_Manual",
    "type_name",         "RTC_RobotMotion_Manual",
    "description",       "ゲームコントローラのデータよりRobotMotion生成　",
    "version",           "1.0.0",
    "vendor",            "AIZUK",
    "category",          "Controller",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.Velocity_Gain", "0.36",
    "conf.default.Omega_Gain", "2.2",
    // Widget
    "conf.__widget__.Velocity_Gain", "text",
    "conf.__widget__.Omega_Gain", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
RTC_RobotMotion_Manual::RTC_RobotMotion_Manual(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_Controller_TypeIn("Controller_Type", m_Controller_Type),
    m_ButtonIn("Button", m_Button),
    m_AnalogIn("Analog", m_Analog),
    m_VelocityOut("Velocity", m_Velocity),
    m_SubCrawlerAngleOut("SubCrawlerAngle", m_SubCrawlerAngle),
    m_ModeOut("Mode", m_Mode),
    m_MOT_MainOut("MOT_Main", m_MOT_Main),
    m_MOT_SubOut("MOT_Sub", m_MOT_Sub),
    m_Scan_ModeOut("Scan_Mode", m_Scan_Mode)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
RTC_RobotMotion_Manual::~RTC_RobotMotion_Manual()
{
}



RTC::ReturnCode_t RTC_RobotMotion_Manual::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("Controller_Type", m_Controller_TypeIn);
  addInPort("Button", m_ButtonIn);
  addInPort("Analog", m_AnalogIn);
  
  // Set OutPort buffer
  addOutPort("Velocity", m_VelocityOut);
  addOutPort("SubCrawlerAngle", m_SubCrawlerAngleOut);
  addOutPort("Mode", m_ModeOut);
  addOutPort("MOT_Main", m_MOT_MainOut);
  addOutPort("MOT_Sub", m_MOT_SubOut);
  addOutPort("Scan_Mode", m_Scan_ModeOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("Velocity_Gain", m_Velocity_Gain, "0.36");
  bindParameter("Omega_Gain", m_Omega_Gain, "2.2");
  // </rtc-template>
    m_SubCrawlerAngle.data.length(11);//.data[0]
    m_Mode.data.length(11);
    m_Analog.data.length(11);
    m_MOT_Main.data.length(11);
    m_MOT_Sub.data.length(11);
    m_Scan_Mode.data.length(11);
    for(int i=0;i<11;i++){
        m_SubCrawlerAngle.data[i]=0;
        m_Mode.data[i]=0;
        m_Scan_Mode.data[i]=0;
        m_Analog.data[i]=0;
        m_MOT_Main.data[i]=0;
        m_MOT_Sub.data[i]=0;
    }
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_RobotMotion_Manual::onFinalize()
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_RobotMotion_Manual::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_RobotMotion_Manual::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_RobotMotion_Manual::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_RobotMotion_Manual::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_RobotMotion_Manual::onExecute(RTC::UniqueId ec_id)
{
  int i=0,thflg=0,quad=0;
 // double Stick_Jud[10];
  double Stick_Jud[10];
  double Vr,Vl,th;
  double wr,wl;
  double Vx,Vy,Va;
  char c[8];
  
/***************************************************************************
/	
/    　Button Read
/
***************************************************************************/
  if(m_ButtonIn.isNew()){
  	m_ButtonIn.read();
 
 	sprintf(c,"%04x",m_Button.data);
 	std::cerr<<"\x1b[1m" <<"\x1b[36m" << c<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
 	for(i=0;i<5;i++){
 		//std::cerr<<"\x1b[1m" <<"\x1b[36m" 
 		//<< i<<"\t"<< c[i]<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;

	  	switch(c[i]){
		      case '1':  //0001
			      button[3+4*(- i + 3) ]=0;
			      button[2+4*(- i + 3) ]=0;
			      button[1+4*(- i + 3) ]=0;
			      button[4*(- i + 3) ]=1;
		      break;
		      
		      case '2':  //0010
			      button[3+4*(- i + 3) ]=0;
			      button[2+4*(- i + 3) ]=0;
			      button[1+4*(- i + 3) ]=1;
			      button[4*(- i + 3) ]=0;
		      break;
		      
		      case '3':  //0011
			      button[3+4*(- i + 3) ]=0;
			      button[2+4*(- i + 3) ]=0;
			      button[1+4*(- i + 3) ]=1;
			      button[4*(- i + 3) ]=1;
		      break;
		      
		      case '4':  //0100
			      button[3+4*(- i + 3) ]=0;
			      button[2+4*(- i + 3) ]=1;
			      button[1+4*(- i + 3) ]=0;
			      button[4*(- i + 3) ]=0;
		      break;
		      
		      case '5':  //0101
			      button[3+4*(- i + 3) ]=0;
			      button[2+4*(- i + 3) ]=1;
			      button[1+4*(- i + 3) ]=0;
			      button[4*(- i + 3) ]=1;
		      break;
		      
		      case '6':  //0110
			      button[3+4*(- i + 3) ]=0;
			      button[2+4*(- i + 3) ]=1;
			      button[1+4*(- i + 3) ]=1;
			      button[4*(- i + 3) ]=0;
		      break;
		      
		      case '7':  //0111
			      button[3+4*(- i + 3) ]=0;
			      button[2+4*(- i + 3) ]=1;
			      button[1+4*(- i + 3) ]=1;
			      button[4*(- i + 3) ]=1;
		      break;
		      
		      case '8':  //1000
			      button[3+4*(- i + 3) ]=1;
			      button[2+4*(- i + 3) ]=0;
			      button[1+4*(- i + 3) ]=0;
			      button[4*(- i + 3) ]=0;
		      break;
		      case '9':  //1001
			      button[3+4*(- i + 3) ]=1;
			      button[2+4*(- i + 3) ]=0;
			      button[1+4*(- i + 3) ]=0;
			      button[4*(- i + 3) ]=1;
		      break;
		      
		      case 'A':  //1010
		      case 'a':  //1010
			      button[3+4*(- i + 3) ]=1;
			      button[2+4*(- i + 3) ]=0;
			      button[1+4*(- i + 3) ]=1;
			      button[4*(- i + 3) ]=0;
		      break;
		      
		      case 'B':  //1011
		      case 'b':  //1011
			      button[3+4*(- i + 3) ]=1;
			      button[2+4*(- i + 3) ]=0;
			      button[1+4*(- i + 3) ]=1;
			      button[4*(- i + 3) ]=1;
		      break;
		      
		      case 'C':  //1100
		      case 'c':  //1100
			      button[3+4*(- i + 3) ]=1;
			      button[2+4*(- i + 3) ]=1;
			      button[1+4*(- i + 3) ]=0;
			      button[4*(- i + 3) ]=0;
		      break;
		      
		      case 'D':  //1101
		      case 'd':  //1101
			      button[3+4*(- i + 3) ]=1;
			      button[2+4*(- i + 3) ]=1;
			      button[1+4*(- i + 3) ]=0;
			      button[4*(- i + 3) ]=1;
		      break;
		      
		      case 'E':  //1110
		      case 'e':  //1110
			      button[3+4*(- i + 3) ]=1;
			      button[2+4*(- i + 3) ]=1;
			      button[1+4*(- i + 3) ]=1;
			      button[4*(- i + 3) ]=0;
		      break;
		      
		      case 'F':  //1111
		      case 'f':  //1111
			      button[3+4*(- i + 3) ]=1;
			      button[2+4*(- i + 3) ]=1;
			      button[1+4*(- i + 3) ]=1;
			      button[4*(- i + 3) ]=1;
		      break;
		      
		      default://0000
			      button[3+4*(- i + 3) ]=0;
			      button[2+4*(- i + 3) ]=0;
			      button[1+4*(- i + 3) ]=0;
			      button[4*(- i + 3) ]=0;
		      break;
	  //num[i]= strtoul(ni_input, &e, 2);

		}
        }
	Button_Rename();
	B_FLG=1;
  }else{
  	for(i=0;i<20;i++){
  		button[i]=0;
  	}
	B_FLG=0;
  }//Button
	
/***************************************************************************
/	
/    　Analog Read
/
***************************************************************************/
  if(m_AnalogIn.isNew()){
	m_AnalogIn.read();
	Analog_Rename();
	A_FLG=1;
  }else{
  	A_FLG=0;
  }
  	
  	
/***************************************************************************
/	
/    　Mode
/
***************************************************************************/
if(B_FLG==1){
  //操作方変更
	if(SQARE){
		if(OLD_SQARE==SQARE){

		}else{
			if(m_Mode.data[0]==1  ){
				m_Mode.data[0]=0;//      TYPE　- A　
			}else{
				m_Mode.data[0]=1;//      TYPE　- B　
			}
		}
	}
	OLD_SQARE=SQARE;
	
	if(PS){
		if(OLD_PS==PS){
			PS_CNT++;
			if(PS_CNT>1000 && PS_LP_Lock==0){
				if(m_Scan_Mode.data[1]==1  ){
					m_Scan_Mode.data[1]=0;//      SCAN_FLG　
					PS_LP_Lock=1;
				}else{
					m_Scan_Mode.data[1]=1;//      SCAN_FLG
					PS_LP_Lock=1;
				}
			}
			
		}else{
			if(m_Scan_Mode.data[0]==1  ){
				m_Scan_Mode.data[0]=0;//      SCAN_FLG　
			}else{
				m_Scan_Mode.data[0]=1;//      SCAN_FLG
			}
			
			PS_LP_Lock=0;
			
			PS_CNT=0;
		}
		
		
	}else{
		PS_LP_Lock=0;
		PS_CNT=0;
	}
	OLD_PS=PS;
	
//速度変更
	if(CIRCLE){
		if(OLD_CIRCLE==CIRCLE){
			
		}else{
			m_Mode.data[1]++;
			
			if(m_Mode.data[1]>8)m_Mode.data[1]=0;//     9段階速度変化
			
		}
	}
  }
	OLD_CIRCLE=CIRCLE;
	
  //最大速度強制
	if(L3){
		m_Mode.data[2]=1;
		}else{
		m_Mode.data[2]=0;
		//}
	}
/***************************************************************************
/	
/    　速度指令生成
/
***************************************************************************/
         //
  	//      TYPE　- A　
  	//
  	if(m_Mode.data[0] == 0 && A_FLG==1){
 	  		for (i=0;i<8;i++){
			//std::cerr<<"\x1b[1m" <<"\x1b[35m" <<i<<":"<< m_Analog.data[i]<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	  		}
	  		//	スティック状態取得 Vl Vr　の値の取りうる範囲 [ 0 〜 100 ] 
	  		//　　PWMの0% 〜 100%として扱う　
	  		// 最大速度の0% 〜 100%として扱う
	  		Stick_Jud[0]= -1*LEFT_X*100;//⇕
	  		Stick_Jud[1]= LEFT_Y*100;//⇔
	  		
	  		Stick_Jud[4]= Stick_Jud[0]-Stick_Jud[1];//⇔
	  		Stick_Jud[5]= Stick_Jud[0]+Stick_Jud[1];//⇔
	  		Stick_Jud[6]= (Stick_Jud[4]+Stick_Jud[5])/2;//⇔
	  		
	  		if(Stick_Jud[6] == 100 || Stick_Jud[6] == -100 ){
	  			Vr=Stick_Jud[6];
	  			Vl=Stick_Jud[6];
	  			th=3.1415926/2;
	  			thflg=1;
		  		if(Stick_Jud[6] == -100){
		  			th=-3.1415926/2;
		  			thflg=-1;
		  		}
	  		}else{
		  		if( Stick_Jud[0] == 0   &&   Stick_Jud[1] == 0 ){//
		  			//std::cerr<<"0"<< std::endl ;
		  			Vr=0;
		  			Vl=0;
		  			th=atan2(Stick_Jud[0],Stick_Jud[1]);
		  			quad=0;
		  			//preV
		  		}
		  		else if( Stick_Jud[0] >= 0     &&  Stick_Jud[0] <= 100    &&    Stick_Jud[1] >= 0     &&  Stick_Jud[1] <= 100 ){//第1象限
		  			//std::cerr<<"1"<< std::endl ;
		  			Vr=Stick_Jud[4];
		  			Vl=Stick_Jud[6]-Vr;
		  			th=atan2(Stick_Jud[0],Stick_Jud[1]);
		  			quad=1;
		  			//preV
		  		}
		  		else if( Stick_Jud[0] >= 0     &&  Stick_Jud[0] <= 100    &&    Stick_Jud[1] >= -100  &&  Stick_Jud[1] <= 0 ){//第2象限
		  			//std::cerr<<"2"<< std::endl ;
		  			Vl=Stick_Jud[5];
		  			Vr=Stick_Jud[6]-Vl;
		  			th=atan2(Stick_Jud[0],Stick_Jud[1]);
		  			quad=2;
		  		}
		  		else if( Stick_Jud[0] >= -100  &&  Stick_Jud[0] <= 0      &&    Stick_Jud[1] >= -100  &&  Stick_Jud[1] <= 0 ){//第3象限
		  			//std::cerr<<"3"<< std::endl ;
		  			Vr=Stick_Jud[4];
		  			Vl=Stick_Jud[6]-Vr;
		  			th=atan2(Stick_Jud[0],Stick_Jud[1]);
		  			quad=3;
		  		}
		  		else if( Stick_Jud[0] >= -100  &&  Stick_Jud[0] <= 0      &&    Stick_Jud[1] >= 0     &&  Stick_Jud[1] <= 100 ){//第4象限
		  			//::cerr<<"4"<< std::endl ;
		  			Vl=Stick_Jud[5];
		  			Vr=Stick_Jud[6]-Vl;
		  			th=atan2(Stick_Jud[0],Stick_Jud[1]);
		  			quad=4;
		  		}
	  		}
	  		
	  		if(m_Mode.data[2]==1){//最大速度モード
	  				Vr*=1.00;
	  				Vl*=1.00;
  			
	  		//	Stick_Jud[0] = -1*m_Analog.data[2]*100;//⇕L
	  		//	Stick_Jud[2] = -1*m_Analog.data[4]*100;//⇕R
  			
  			}else{
	  			
	  			if(m_Mode.data[1]==0){
	  				Vr*=0.20;
	  				Vl*=0.20;
	  			}else if(m_Mode.data[1]==1){
	  				Vr*=0.30;
	  				Vl*=0.30;
	  			}else if(m_Mode.data[1]==2){
	  				Vr*=0.40;
	  				Vl*=0.40;
	  			}else if(m_Mode.data[1]==3){
	  				Vr*=0.50;
	  				Vl*=0.50;
	  			}else if(m_Mode.data[1]==4){
	  				Vr*=0.60;
	  				Vl*=0.60;
	  			}else if(m_Mode.data[1]==5){
	  				Vr*=0.70;
	  				Vl*=0.70;
	  			}else if(m_Mode.data[1]==6){
	  				Vr*=0.80;
	  				Vl*=0.80;
	  			}else if(m_Mode.data[1]==7){
	  				Vr*=0.90;
	  				Vl*=0.90;
	  			}else if(m_Mode.data[1]==8){
	  				Vr*=1.00;
	  				Vl*=1.00;
	  			}else{
	  			}
	  			
	  			
		  	}
			std::cerr<<"\x1b[1m" <<"\x1b[34m" << "Vr:"<<Vr << " Vl:"<<Vl<< " th:"<<th*degree<<
			 " cos:"<<sin(th)*Vr*m_Velocity_Gain*0.01<<
			  " cos*VR:"<<cos(th)*Vr*m_Omega_Gain*0.01<<
			  " Quad:"<<quad<<
			  " jud0:"<<Stick_Jud[0]<<
			  " jud1:"<<Stick_Jud[1]<<
			   
			 "\x1b[39m"<<"\x1b[0m"<< std::endl ;
	  		m_MOT_Main.data[0]=Vr;
	  		m_MOT_Main.data[1]=Vl;
	  		m_MOT_MainOut.write();
		
			/*
			if(Vr>=90){
				Vr=90;
			}
			
			if(Vr<=-90){
				Vr=-90;
			}
			
			if(Vl>=90){
				Vl=90;
			}
			if(Vl<=-90){
				Vl=-90;
			}
			if(Vr<=10 && Vr>=0) Vr=10;
			if(Vr>=-10 && Vr<0) Vr=-10;			
//			if(Vr==0) Vr=0;
			if(Vl<=10 && Vl>=0) Vl=10;
			if(Vl>=-10 && Vl<0) Vl=-10;
			*/
			//
			//   | 　v |    | Rw/2  Rw/2 　| | wr |
			//   |　   | =  |      　　　　　      | |   　 |
			//  　|　 w |    | Rw/T  Rw/T | | wl |
			//
			//   v:機体速度 m/s  w：機体角速度 rad/s
			//   Rw:車輪半径 m　　 T:車輪間幅 m
			//   wr:右車輪 rad/s wr:左車輪 rad/s
			//if(Vr>=0)wr=118.75*Vr-1187.5;//pwm→rpm
			//else if(Vr<0)wr=118.75*Vr+1187.5;//pwm→rpm
			/*
			if(Vr>=0)wr=71*Vr-710;//pwm→rpm
			else if(Vr<0)wr=71*Vr+710;//pwm→rpm
			wr=wr*6.283/60/135;//rad/s
			
			//if(Vl>=0)wl=118.75*Vl-1187.5;//pwm→rpm
			//else if(Vl<0)wl=118.75*Vl+1187.5;//pwm→rpm
			if(Vl>=0)wl=71*Vl-710;//pwm→rpm
			else if(Vl<0)wl=71*Vl+710;//pwm→rpm
			wl=wl*6.283/60/135;//rad/s
			*/
			
			if(Vr>=0&&Vr<=20)wr=0.00356*Vr-0.0356;//pwm→rpm
			else if(Vr<0&&Vr>=-20)wr=0.00356*Vr+0.0356;//pwm→rpm 0.00356	0.0356
			
			else if(Vr>20&&Vr<80)wr=0.00456*Vr-0.0543;//pwm→rpm 0.00456	0.0543

			else if(Vr<-20&&Vr>-80)wr=0.00456*Vr+0.0543;//pwm→rpm 0.00356	0.0356
			
			else if(Vr>=80&&Vr<=90)wr=0.00259*Vr+0.1017;//pwm→rpm 0.00259	0.1017

			else if(Vr<80&&Vr>=-90)wr=0.00259*Vr-0.1017;//pwm→rpm 0.00356	0.0356
			
			
			if(Vl>=0&&Vl<=20)wl=0.00356*Vl-0.0356;//pwm→rpm
			else if(Vl<0&&Vl>=-20)wl=0.00356*Vl+0.0356;//pwm→rpm 0.00356	0.0356
			
			else if(Vl>20&&Vl<80)wl=0.00456*Vl-0.0543;//pwm→rpm 0.00456	0.0543

			else if(Vl<-20&&Vl>-80)wl=0.00456*Vl+0.0543;//pwm→rpm 0.00356	0.0356
			
			else if(Vl>=80&&Vl<=90)wl=0.00259*Vl+0.1017;//pwm→rpm 0.00259	0.1017

			else if(Vl<80&&Vl>=-90)wl=0.00259*Vl-0.1017;//pwm→rpm 0.00356	0.0356
		//	std::cerr<<"\x1b[1m" <<"\x1b[34m" << "wr:"<<wr << " wl:"<<wl<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;

		//	
double Rw=0.07681;
double T=0.3288;
int sign[2];
			
			
			Vx=Rw/2*wr+Rw/2*wl;//v
			
			Vy=0;
			Va=-(Rw/T*wr-Rw/T*wl);
			
			Vx=(wr+wl)/2;//v
			Vy=0;
			Va=(wr-wl)/T;
			std::cerr<<"\x1b[1m" <<"\x1b[34m" << "VxW:"<<Vx << " VaW:"<<Va<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
			
			
			if (Stick_Jud[0]>0)sign[0]=1;
			if (Stick_Jud[0]<0)sign[0]=-1;
			if (Stick_Jud[1]>0)sign[1]=1;
			if (Stick_Jud[1]<0)sign[1]=-1;
			
			Vx=sin(th)*Vr*m_Velocity_Gain*0.01*sign[0];//v
			Vy=0;
			Va=cos(th)*Vr*m_Omega_Gain*0.01*sign[1];
			
			std::cerr<<"\x1b[1m" <<"\x1b[34m" << "Vx:"<<Vx << " Va:"<<Va<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
			
			double pwm_R;
			double pwm_L;
		//	pwm_R=Vx/Rw+Va*T/(2*Rw);
		//	pwm_L=Vx/Rw-Va*T/(2*Rw);			
			//if(pwm_R>=0)pwm_R=(int)((pwm_R*60*135/6.283)*0.00842+10.5);
			//else if(pwm_R<0)pwm_R=(int)((pwm_R*60*135/6.283)*0.00842-10.5);
			if(pwm_R>=0)pwm_R=(int)((pwm_R*60*135/6.283)*0.01408+10.5);
			else if(pwm_R<0)pwm_R=(int)((pwm_R*60*135/6.283)*0.01408-10.5);
			std::cerr<<"\x1b[1m" <<"\x1b[34m" << "pwm"<<pwm_R<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
			
	  		m_Velocity.data.vx=Vx;
	  		m_Velocity.data.vy=Vy;
	  		m_Velocity.data.va=Va;
			m_VelocityOut.write();	
					
		}
  	//
  	//      TYPE　- B
  	//
  		else if(m_Mode.data[0] == 1   && A_FLG == 1  )  {//TYPEB m_Mode.dataStick_Jud[0]= -1*LEFT_X*100;//⇕

	  		Vr = -1*LEFT_X*100;//⇕L
	  		Vl = -1*RIGHT_X*100;//⇕R
	  		
	  	
	  			
	  			if(m_Mode.data[1]==0){
	  				Vr*=0.20;
	  				Vl*=0.20;
	  			}else if(m_Mode.data[1]==1){
	  				Vr*=0.30;
	  				Vl*=0.30;
	  			}else if(m_Mode.data[1]==2){
	  				Vr*=0.40;
	  				Vl*=0.40;
	  			}else if(m_Mode.data[1]==3){
	  				Vr*=0.50;
	  				Vl*=0.50;
	  			}else if(m_Mode.data[1]==4){
	  				Vr*=0.60;
	  				Vl*=0.60;
	  			}else if(m_Mode.data[1]==5){
	  				Vr*=0.70;
	  				Vl*=0.70;
	  			}else if(m_Mode.data[1]==6){
	  				Vr*=0.80;
	  				Vl*=0.80;
	  			}else if(m_Mode.data[1]==7){
	  				Vr*=0.90;
	  				Vl*=0.90;
	  			}else if(m_Mode.data[1]==8){
	  				Vr*=1.00;
	  				Vl*=1.00;
	  			}else{
	  			}
	  			
	  			if(L3)Vl*=1.00;
		  	
	  			if(R3)Vr*=1.00;
	  			
	  		m_MOT_Main.data[0]=Vr;
	  		m_MOT_Main.data[1]=Vl;
	  		m_MOT_MainOut.write();
		std::cerr<<"\x1b[1m" <<"\x1b[34m" << "Vr:"<<Vr << " Vl:"<<Vl<< " th:"<<th*degree<<
			  " jud0:"<<Stick_Jud[0]<<
			  " jud1:"<<Stick_Jud[1]<<
			   
			 "\x1b[39m"<<"\x1b[0m"<< std::endl ;
	  	
	  	Vx=(Vr+Vl)/2*m_Velocity_Gain*0.01;//v
		Vy=0;
		Va=-1*(Vr-Vl)/2*m_Omega_Gain*0.01;
  		m_Velocity.data.vx=Vx;
  		m_Velocity.data.vy=Vy;
  		m_Velocity.data.va=Va;
		m_VelocityOut.write();	
		std::cerr<<"\x1b[1m" <<"\x1b[34m" << "Vx:"<<Vx << " Va:"<<Va<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
		
	}
	  		
  
  
  
  	

////////////////////////////////////////////////////////////////////////////
//
//Subモータ
//      動作部位指定
//		L1ボタン	左前サブ	R1ボタン　右前サブ
//		L２ボタン 左後サブ	R2ボタン　右後サブ
//	
//      動作方向指定
//		△ボタン　機体内側へ
//		✕ボタン 　機体外側へ
//
////////////////////////////////////////////////////////////////////////////
	if(B_FLG==1){
		//動作方向決定
		if(TRIANGLE){		
			//動作部位決定
			if(L1){
	  			m_MOT_Sub.data[1]=-90;
	
			}else{
	  			m_MOT_Sub.data[1]=0;
			}

			if(L2){
	  			m_MOT_Sub.data[3]=90;
			}else{
	  			m_MOT_Sub.data[3]=10;
			}

			if(R1){
	  			m_MOT_Sub.data[0]=-90;
			}else{
	  			m_MOT_Sub.data[0]=10;
			}

			if(R2){
	  			m_MOT_Sub.data[2]=90;
			}else{
	  			m_MOT_Sub.data[2]=10;
			}
	  		m_MOT_SubOut.write();
		}else if(CROSS){
		//動作部位決定
			if(L1){
	  			m_MOT_Sub.data[1]=90;
	
			}else{
	  			m_MOT_Sub.data[1]=10;
			}

			if(L2){
	  			m_MOT_Sub.data[3]=-90;
			}else{
	  			m_MOT_Sub.data[3]=10;
			}

			if(R1){
	  			m_MOT_Sub.data[0]=90;
			}else{
	  			m_MOT_Sub.data[0]=10;
			}

			if(R2){
	  			m_MOT_Sub.data[2]=-90;
			}else{
	  			m_MOT_Sub.data[2]=10;
			}
	  		m_MOT_SubOut.write();
		
		}else{
	  		m_MOT_Sub.data[0]=0;
	  		m_MOT_Sub.data[1]=0;
	  		m_MOT_Sub.data[2]=0;
	  		m_MOT_Sub.data[3]=0;
	  		m_MOT_SubOut.write();
		}
	
		m_SubCrawlerAngle.data[0]=555;
		m_SubCrawlerAngle.data[1]=555;
		m_SubCrawlerAngle.data[2]=555;
		m_SubCrawlerAngle.data[3]=555;

	
		if(HAT_UP||HAT_DOWN||HAT_RIGHT||HAT_LEFT){
			if(L1){
	  			if(HAT_UP)m_SubCrawlerAngle.data[1]=90;
	  			if(HAT_DOWN)m_SubCrawlerAngle.data[1]=-90;
	  			if(HAT_RIGHT)m_SubCrawlerAngle.data[1]=45;
	  			if(HAT_LEFT)m_SubCrawlerAngle.data[1]=0;
				m_Mode.data[3]=1;
			}else{
	  			m_MOT_Sub.data[1]=0;
			}

			if(L2){
	  			if(HAT_UP)m_SubCrawlerAngle.data[3]=90;
	  			if(HAT_DOWN)m_SubCrawlerAngle.data[3]=-90;
	  			if(HAT_RIGHT)m_SubCrawlerAngle.data[3]=45;
	  			if(HAT_LEFT)m_SubCrawlerAngle.data[3]=0;
	  			m_Mode.data[3]=1;
			}else{
	  			m_MOT_Sub.data[3]=0;
			}

			if(R1){
	  			if(HAT_UP)m_SubCrawlerAngle.data[0]=90;
	  			if(HAT_DOWN)m_SubCrawlerAngle.data[0]=-90;
	  			if(HAT_RIGHT)m_SubCrawlerAngle.data[0]=45;
	  			if(HAT_LEFT)m_SubCrawlerAngle.data[0]=0;
	  			m_Mode.data[3]=1;
			}else{
	  			m_MOT_Sub.data[0]=0;
			}

			if(R2){
	  			if(HAT_UP)m_SubCrawlerAngle.data[2]=90;
	  			if(HAT_DOWN)m_SubCrawlerAngle.data[2]=-90;
	  			if(HAT_RIGHT)m_SubCrawlerAngle.data[2]=45;
	  			if(HAT_LEFT)m_SubCrawlerAngle.data[2]=0;
	  			m_Mode.data[3]=1;
			}else{
	  			m_MOT_Sub.data[2]=0;
			}
			}else{  
				m_SubCrawlerAngle.data[0]=555;
	  			m_SubCrawlerAngle.data[1]=555;
	 			m_SubCrawlerAngle.data[2]=555;
	  			m_SubCrawlerAngle.data[3]=555;
	  			m_Mode.data[3]=0;
			}
			m_SubCrawlerAngleOut.write();
	//	  	std::cerr<<"\x1b[1m" <<"\x1b[35m" <<m_SubCrawlerAngle.data[0]<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	//	  	std::cerr<<"\x1b[1m" <<"\x1b[35m" <<m_SubCrawlerAngle.data[1]<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	//	  	std::cerr<<"\x1b[1m" <<"\x1b[35m" <<m_SubCrawlerAngle.data[2]<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	//	  	std::cerr<<"\x1b[1m" <<"\x1b[35m" <<m_SubCrawlerAngle.data[3]<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
		 	std::cerr<<"\x1b[1m" <<"\x1b[35m" <<m_Mode.data[0]<<m_Mode.data[1]<<m_Mode.data[2]<<m_Mode.data[3]<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
		 	std::cerr<<"\x1b[1m" <<"\x1b[35m" <<m_SubCrawlerAngle.data[0]<<":"<<m_SubCrawlerAngle.data[1]<<":"<<m_SubCrawlerAngle.data[2]<<":"<<m_SubCrawlerAngle.data[3]<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
			m_ModeOut.write();
	  		m_Scan_ModeOut.write();
	  	
  	}
	  
	  
  
  
  
  
	
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_RobotMotion_Manual::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_RobotMotion_Manual::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_RobotMotion_Manual::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_RobotMotion_Manual::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_RobotMotion_Manual::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

RTC::ReturnCode_t RTC_RobotMotion_Manual::Button_Rename()
{
	SQARE=button[0];
	CROSS=button[1];
	CIRCLE=button[2];
	TRIANGLE=button[3];
	L1=button[4];
	R1=button[5];
	L2=button[6];
	R2=button[7];
	SHARE=button[8];
	OPTION=button[9];
	L3=button[10];
	R3=button[11];
	PS=button[12];
	Touch=button[13];
#ifdef _DEBUG
	if(SQARE)
  	std::cerr<<"\x1b[1m" <<"\x1b[35m" << "□"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(CROSS)
  	std::cerr<<"\x1b[1m" <<"\x1b[36m" << "✕"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(CIRCLE)
  	std::cerr<<"\x1b[1m" <<"\x1b[31m" << "○"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(TRIANGLE)
  	std::cerr<<"\x1b[1m" <<"\x1b[32m" << "△"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(L1)
  	std::cerr<<"\x1b[1m" <<"\x1b[37m" << "L1"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(R1)
  	std::cerr<<"\x1b[1m" <<"\x1b[37m" << "R1"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(L2)
  	std::cerr<<"\x1b[1m" <<"\x1b[37m" << "L2"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(R2)
  	std::cerr<<"\x1b[1m" <<"\x1b[37m" << "L2"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(SHARE)
  	std::cerr<<"\x1b[1m" <<"\x1b[34m" << "SHARE"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(OPTION)
  	std::cerr<<"\x1b[1m" <<"\x1b[35m" << "OPTION"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(L3)
  	std::cerr<<"\x1b[1m" <<"\x1b[37m" << "L3"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(R3)
  	std::cerr<<"\x1b[1m" <<"\x1b[37m" << "R3"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(PS)
  	std::cerr<<"\x1b[1m" <<"\x1b[35m" << "PS"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(Touch)
  	std::cerr<<"\x1b[1m" <<"\x1b[36m" << "Touch"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
#endif
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_RobotMotion_Manual::Analog_Rename()
{

	if(m_Analog.data[0]==-1){
		HAT_UP=1;
		HAT_DOWN=0;
	}else if(m_Analog.data[0]==1){
	 	HAT_UP=0;
	 	HAT_DOWN=1;
	}else{
		HAT_UP=0;
		HAT_DOWN=0;
	}
	if(m_Analog.data[1]==-1){
		HAT_RIGHT=0;
		HAT_LEFT=1;
	}else if(m_Analog.data[1]==1){
	 	HAT_RIGHT=1;
	 	HAT_LEFT=0;
	}else{
		HAT_RIGHT=0;
		HAT_LEFT=0;
	}

	LEFT_X=m_Analog.data[2];
	LEFT_Y=m_Analog.data[3];
	RIGHT_X=m_Analog.data[4];
	RIGHT_Y=m_Analog.data[5];
	L2_A=m_Analog.data[6];
	R2_A=m_Analog.data[7];
#ifdef _DEBUG
	if(HAT_UP)
  	std::cerr<<"\x1b[1m" <<"\x1b[35m" << "↑"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(HAT_DOWN)
  	std::cerr<<"\x1b[1m" <<"\x1b[36m" << "↓"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(HAT_RIGHT)
  	std::cerr<<"\x1b[1m" <<"\x1b[31m" << "→"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	if(HAT_LEFT)
  	std::cerr<<"\x1b[1m" <<"\x1b[32m" << "←"<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	//L_X
	std::cerr<<"\x1b[1m" <<"\x1b[37m" << LEFT_X<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	//L_Y
  	std::cerr<<"\x1b[1m" <<"\x1b[37m" <<LEFT_Y<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	//R_X
  	std::cerr<<"\x1b[1m" <<"\x1b[37m" <<RIGHT_X<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	//R_Y
  	std::cerr<<"\x1b[1m" <<"\x1b[37m" << RIGHT_Y<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	//L2
  	std::cerr<<"\x1b[1m" <<"\x1b[34m" <<L2_A<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	//R2
  	std::cerr<<"\x1b[1m" <<"\x1b[35m" << R2_A<<"\x1b[39m"<<"\x1b[0m"<< std::endl ;
	
#endif
  return RTC::RTC_OK;
}
extern "C"
{
 
  void RTC_RobotMotion_ManualInit(RTC::Manager* manager)
  {
    coil::Properties profile(rtc_robotmotion_manual_spec);
    manager->registerFactory(profile,
                             RTC::Create<RTC_RobotMotion_Manual>,
                             RTC::Delete<RTC_RobotMotion_Manual>);
  }
  
};


